/*
** OpenBus Demo - Orbix 6.3
** server.cpp
*/

#include <fstream>
#include <iostream>
#include <omg/orb.hh>
#include <it_ts/termination_handler.h>

#include <ComponentBuilderOrbix.h>
#include <IComponentOrbix.h>
#include <IMetaInterfaceOrbix.h>

#include <stubs/scs.hh>

#include "stubs/helloS.hh"

using namespace std;

IT_USING_NAMESPACE_STD

scs::core::ComponentContext* componentContext = NULL;

class HelloImpl : virtual public POA_demoidl::hello::IHello {
  private:
    scs::core::ComponentContext* componentContext;
  public:
    HelloImpl(scs::core::ComponentContext* componentContext) {
      this->componentContext = componentContext;
    }
    static void* instantiate(scs::core::ComponentContext* componentContext) {
      return (void*) new HelloImpl(componentContext);
    }
    static void destruct(void* obj) {
      delete (HelloImpl*) obj;
    }
    void sayHello() IT_THROW_DECL((CORBA::SystemException)) {
      cout << "Servant diz: HELLO!" << endl;
    };
};

static void myTerminationHandler(long signal) {
  cout << "Encerrando o processo servidor..." << endl;
  try {
  } catch(CORBA::Exception& e) {
    cout << "No foi possvel remover a oferta de servio." << endl;
  }
  delete componentContext;
}

int main(int argc, char* argv[]) {
  #ifdef VERBOSE
    Verbose* verbose = new Verbose();
    scs::core::ComponentBuilder::verbose = verbose;
    scs::core::IComponentImpl::verbose = verbose;
    scs::core::IMetaInterfaceImpl::verbose = verbose;
  #endif

  IT_TerminationHandler termination_handler(myTerminationHandler);

  CORBA::ORB_var orb = CORBA::ORB::_nil();;
  PortableServer::POA_var poa = PortableServer::POA::_nil();
  PortableServer::POAManager_var poa_manager;
    
  orb = CORBA::ORB_init(argc, argv);
  CORBA::Object_var poa_obj = orb->resolve_initial_references("RootPOA");
  poa = PortableServer::POA::_narrow(poa_obj);
  poa_manager = poa->the_POAManager();
  poa_manager->activate();

/* Definio do componente. */
  scs::core::ComponentId componentId;
  componentId.name = "HelloComponent";
  componentId.major_version = '1';
  componentId.minor_version = '0';
  componentId.patch_version = '0';
  componentId.platform_spec = "nenhuma";

/* Fbrica de componentes */
  scs::core::ComponentBuilder* componentBuilder = new scs::core::ComponentBuilder(orb, poa);

/* Contexto do componente compartilhado entre todas as facetas */
  componentContext = new scs::core::ComponentContext(componentBuilder, &componentId);

/* Descrio das facetas. */
  std::list<scs::core::ExtendedFacetDescription> extFacets;
  scs::core::ExtendedFacetDescription helloDesc;
  helloDesc.name = "IHello";
  helloDesc.interface_name = "IDL:demoidl/hello/IHello:1.0";
  helloDesc.instantiator = HelloImpl::instantiate;
  helloDesc.destructor = HelloImpl::destruct;
  extFacets.push_back(helloDesc);
  
/* Instanciacao das facetas manualmente (evita um problema na Solaris 10 SPARC) */
  std::list<PortableServer::ServantBase*> facetInstances;
  facetInstances.push_back((PortableServer::ServantBase*) new HelloImpl(componentContext));

  componentContext = componentBuilder->newComponent(extFacets, componentId, &facetInstances, componentContext);

  scs::core::FacetDescription* f = NULL;
  std::map<std::string, scs::core::FacetDescription>::iterator it;
  std::map<std::string, scs::core::FacetDescription> facetDescs = componentContext->getFacetDescs();
  for (it = facetDescs.begin(); it != facetDescs.end(); it++) {
    f = &(*it).second;

    char* ior = orb->object_to_string(f->facet_ref);
    std::cout << "Gravando o IOR da faceta "<< (*it).first <<" num respectivo arquivo .ior..." << std::endl; 
    std::string filename;
    filename += (*it).first;
    filename += ".ior";

    ofstream os(filename.c_str());
    os << ior; 
    os.close();
    free(ior);
  }
    
  cout << "Servio IHello registrado." << endl;
  cout << "Aguardando requisies..." << endl;

  orb->run();

  return 0;
}

